@@ -133,7 +133,7 @@ public class SwiftNativeFileSystemStore {
                              InputStream inputStream, long length)
           throws IOException {
 
-    String stringPath = path.toUri().getPath();
+    String stringPath = path.toUri().toString();
     String partitionFilename = SwiftUtils.partitionFilenameFromNumber(
       partNumber);
     if (stringPath.endsWith("/")) {
@@ -210,40 +210,15 @@ public class SwiftNativeFileSystemStore {
     throws IOException, FileNotFoundException {
 
     SwiftObjectPath objectPath = toObjectPath(path);
-
-    // remove trailing slash because FileStatus must not include that
-    Path statusPath = path;
-    if (statusPath.toUri().toString().endsWith("/")) {
-      String pathUri = statusPath.toUri().toString();
-      if (pathUri.length() > 1)
-        statusPath = new Path(pathUri.substring(0, pathUri.length() - 1));
-    }
-
-    Header[] headers = null;
-    try {
-      headers = stat(objectPath, newest);
-    } catch (FileNotFoundException e) {
-      try {
-        // retry stat for direcotry file
-        objectPath = toDirPath(path);
-        headers = stat(objectPath, newest);
-      } catch (FileNotFoundException ex) {
-        // if path is pseudo-directory, ignore FileNotFoundException.
-      }
-    }
-    //no headers is treated as a missing file or pseudo-directory
-    if (headers == null || headers.length == 0) {
-      if (existsPseudoDirectory(objectPath)) {
-        Path pseudoDirPath = getCorrectSwiftPath(statusPath);
-        return SwiftFileStatus.createPseudoDirStatus(pseudoDirPath);
-      }
+    final Header[] headers = stat(objectPath, newest);
+    //no headers is treated as a missing file
+    if (headers.length == 0) {
       throw new FileNotFoundException("Not Found " + path.toUri());
     }
 
     boolean isDir = false;
     long length = 0;
     long lastModified = 0 ;
-    SwiftObjectPath dloPrefix = null;
     for (Header header : headers) {
       String headerName = header.getName();
       if (headerName.equals(SwiftProtocolConstants.X_CONTAINER_OBJECT_COUNT) ||
@@ -255,35 +230,25 @@ public class SwiftNativeFileSystemStore {
         length = Long.parseLong(header.getValue());
       }
       if (SwiftProtocolConstants.HEADER_LAST_MODIFIED.equals(headerName)) {
-        final SimpleDateFormat simpleDateFormat = new SimpleDateFormat(PATTERN, Locale.US);
+        final SimpleDateFormat simpleDateFormat = new SimpleDateFormat(PATTERN);
         try {
           lastModified = simpleDateFormat.parse(header.getValue()).getTime();
         } catch (ParseException e) {
           throw new SwiftException("Failed to parse " + header.toString(), e);
         }
       }
-      if (headerName.equals(SwiftProtocolConstants.X_OBJECT_MANIFEST)) {
-        String[] values = header.getValue().split("/", 2);
-        if (values.length == 2) {
-          dloPrefix = new SwiftObjectPath(values[0], "/" + values[1]);
-        }
-      }
     }
     if (lastModified == 0) {
       lastModified = System.currentTimeMillis();
     }
-    if (objectPath.toString().endsWith("/")) {
-      isDir = true;
-    }
 
-    Path correctSwiftPath = getCorrectSwiftPath(statusPath);
+    Path correctSwiftPath = getCorrectSwiftPath(path);
     return new SwiftFileStatus(length,
                                isDir,
                                1,
                                getBlocksize(),
                                lastModified,
-                               correctSwiftPath,
-                               dloPrefix);
+                               correctSwiftPath);
   }
 
   private Header[] stat(SwiftObjectPath objectPath, boolean newest) throws
@@ -299,39 +264,6 @@ public class SwiftNativeFileSystemStore {
     return headers;
   }
 
-  private boolean existsPseudoDirectory(SwiftObjectPath path) {
-    try {
-      String pseudoDirName = path.getObject();
-      if (pseudoDirName.endsWith("/")) {
-        String obj = path.getObject();
-        path = new SwiftObjectPath(path.getContainer(),
-                                   obj.substring(0, obj.length() - 1));
-      } else {
-        pseudoDirName = pseudoDirName.concat("/");
-      }
-
-      final byte[] bytes;
-      bytes = swiftRestClient.listDeepObjectsInDirectory(path, false, false);
-
-      final CollectionType collectionType = JSONUtil.getJsonMapper().
-        getTypeFactory().constructCollectionType(List.class,
-                                                 SwiftObjectFileStatus.class);
-
-      final List<SwiftObjectFileStatus> fileStatusList =
-        JSONUtil.toObject(new String(bytes), collectionType);
-
-      for (SwiftObjectFileStatus status : fileStatusList) {
-        if (pseudoDirName.equals(status.getSubdir())) {
-          return true;
-        }
-      }
-      return false;
-
-    } catch (Exception e) {
-      return false;
-    }
-  }
-
   /**
    * Get the object as an input stream
    *
@@ -469,22 +401,18 @@ public class SwiftNativeFileSystemStore {
    * @param path working path
    * @param listDeep ask for all the data
    * @param newest ask for the newest data
-   * @param addTrailingSlash should a trailing slash be added if there isn't one
    * @return Collection of file statuses
    * @throws IOException IO problems
    * @throws FileNotFoundException if the path does not exist
    */
   private List<FileStatus> listDirectory(SwiftObjectPath path,
                                          boolean listDeep,
-                                         boolean newest,
-                                         boolean addTrailingSlash)
-      throws IOException {
+                                         boolean newest) throws IOException {
     final byte[] bytes;
     final ArrayList<FileStatus> files = new ArrayList<FileStatus>();
     final Path correctSwiftPath = getCorrectSwiftPath(path);
     try {
-      bytes = swiftRestClient.listDeepObjectsInDirectory(path, listDeep,
-                                                         addTrailingSlash);
+      bytes = swiftRestClient.listDeepObjectsInDirectory(path, listDeep);
     } catch (FileNotFoundException e) {
       if (LOG.isDebugEnabled()) {
         LOG.debug("" +
@@ -540,32 +468,14 @@ public class SwiftNativeFileSystemStore {
       return files;
     }
 
-    String pathWithSlash = path.getObject();
-    if (!pathWithSlash.endsWith("/")) {
-      pathWithSlash = pathWithSlash.concat("/");
-    }
-    String prevObjName = "";
     for (SwiftObjectFileStatus status : fileStatusList) {
-      String name = status.getName();
-      if (name == null) {
-        name = status.getSubdir();
-      }
-      if (name == null || name.equals(pathWithSlash)) {
-        continue;
-      }
-
-      if (!name.endsWith("/")) {
-        final Path filePath = getCorrectSwiftPath(new Path(name));
-        files.add(getObjectMetadata(filePath, newest));
-        prevObjName = name;
-      } else {
-        if (prevObjName.length() + 1 == name.length() &&
-            name.startsWith(prevObjName)) {
-          // Ignore because this directory object is stored DLO segment file
-          continue;
-        }
-        final Path dirPath = getCorrectSwiftPath(toDirPath(new Path(name)));
-        files.add(getObjectMetadata(dirPath, newest));
+      if (status.getName() != null) {
+          files.add(new SwiftFileStatus(status.getBytes(),
+                  status.getBytes() == 0,
+                  1,
+                  getBlocksize(),
+                  status.getLast_modified().getTime(),
+                  getCorrectSwiftPath(new Path(status.getName()))));
       }
     }
 
@@ -588,7 +498,7 @@ public class SwiftNativeFileSystemStore {
                                    boolean recursive,
                                    boolean newest) throws IOException {
     final Collection<FileStatus> fileStatuses;
-    fileStatuses = listDirectory(toDirPath(path), recursive, newest, true);
+    fileStatuses = listDirectory(toDirPath(path), recursive, newest);
     return fileStatuses.toArray(new FileStatus[fileStatuses.size()]);
   }
 
@@ -617,7 +527,7 @@ public class SwiftNativeFileSystemStore {
 
   private SwiftObjectPath toDirPath(Path path) throws
           SwiftConfigurationException {
-    return SwiftObjectPath.fromPath(uri, path, true);
+    return SwiftObjectPath.fromPath(uri, path, false);
   }
 
   private SwiftObjectPath toObjectPath(Path path) throws
@@ -644,17 +554,12 @@ public class SwiftNativeFileSystemStore {
   /**
    * deletes object from Swift
    *
-   * @param status FileStatus to delete
+   * @param path path to delete
    * @return true if the path was deleted by this specific operation.
    * @throws IOException on a failure
    */
-  public boolean deleteObject(FileStatus status) throws IOException {
-    SwiftObjectPath swiftObjectPath;
-    if (status.isDir()) {
-      swiftObjectPath = toDirPath(status.getPath());
-    } else {
-      swiftObjectPath = toObjectPath(status.getPath());
-    }
+  public boolean deleteObject(Path path) throws IOException {
+    SwiftObjectPath swiftObjectPath = toObjectPath(path);
     if (!SwiftUtils.isRootDir(swiftObjectPath)) {
       return swiftRestClient.delete(swiftObjectPath);
     } else {
@@ -665,6 +570,18 @@ public class SwiftNativeFileSystemStore {
     }
   }
 
+  /**
+   * deletes a directory from Swift. This is not recursive
+   *
+   * @param path path to delete
+   * @return true if the path was deleted by this specific operation -or
+   *         the path was root and not acted on.
+   * @throws IOException on a failure
+   */
+  public boolean rmdir(Path path) throws IOException {
+    return deleteObject(path);
+  }
+
   /**
    * Does the object exist
    *
@@ -718,32 +635,22 @@ public class SwiftNativeFileSystemStore {
     }
     boolean renamingOnToSelf = src.equals(dst);
 
-    final SwiftFileStatus srcMetadata;
-    srcMetadata = getObjectMetadata(src);
-    SwiftObjectPath srcObject;
-    if (srcMetadata.isDirectory()) {
-      srcObject = toDirPath(src);
-    } else {
-      srcObject = toObjectPath(src);
-    }
+    SwiftObjectPath srcObject = toObjectPath(src);
+    SwiftObjectPath destObject = toObjectPath(dst);
+
     if (SwiftUtils.isRootDir(srcObject)) {
       throw new SwiftOperationFailedException("cannot rename root dir");
     }
 
+    final SwiftFileStatus srcMetadata;
+    srcMetadata = getObjectMetadata(src);
     SwiftFileStatus dstMetadata;
-    SwiftObjectPath destObject;
     try {
       dstMetadata = getObjectMetadata(dst);
-      if (dstMetadata.isDirectory()) {
-        destObject = toDirPath(dst);
-      } else {
-        destObject = toObjectPath(dst);
-      }
     } catch (FileNotFoundException e) {
       //destination does not exist.
       LOG.debug("Destination does not exist");
       dstMetadata = null;
-      destObject = toObjectPath(dst);
     }
 
     //check to see if the destination parent directory exists
@@ -763,11 +670,14 @@ public class SwiftNativeFileSystemStore {
     }
 
     boolean destExists = dstMetadata != null;
-    boolean destIsDir = destExists && dstMetadata.isDirectory();
+    boolean destIsDir = destExists && SwiftUtils.isDirectory(dstMetadata);
     //calculate the destination
     SwiftObjectPath destPath;
 
-    if (srcMetadata.isFile()) {
+    //enum the child entries and everything underneath
+    List<FileStatus> childStats = listDirectory(srcObject, true, true);
+    boolean srcIsFile = !srcMetadata.isDir();
+    if (srcIsFile) {
 
       //source is a simple file OR a partitioned file
       // outcomes:
@@ -794,8 +704,25 @@ public class SwiftNativeFileSystemStore {
         //outcome #3 -new entry
         destPath = toObjectPath(dst);
       }
+      int childCount = childStats.size();
+      //here there is one of:
+      // - a single object ==> standard file
+      // ->
+      if (childCount == 0) {
+        copyThenDeleteObject(srcObject, destPath);
+      } else {
+        //do the copy
+        SwiftUtils.debug(LOG, "Source file appears to be partitioned." +
+                              " copying file and deleting children");
 
-      copyThenDeleteObject(srcObject, srcMetadata, destPath);
+        copyObject(srcObject, destPath);
+        for (FileStatus stat : childStats) {
+          SwiftUtils.debug(LOG, "Deleting partitioned file %s ", stat);
+          deleteObject(stat.getPath());
+        }
+
+        swiftRestClient.delete(srcObject);
+      }
     } else {
 
       //here the source exists and is a directory
@@ -819,7 +746,7 @@ public class SwiftNativeFileSystemStore {
         // #3 destination doesn't exist: create a new dir with that name
         targetPath = dst;
       }
-      SwiftObjectPath targetObjectPath = toDirPath(targetPath);
+      SwiftObjectPath targetObjectPath = toObjectPath(targetPath);
       //final check for any recursive operations
       if (srcObject.isEqualToOrParentOf(targetObjectPath)) {
         //you can't rename a directory onto itself
@@ -830,8 +757,6 @@ public class SwiftNativeFileSystemStore {
 
       LOG.info("mv  " + srcObject + " " + targetPath);
 
-      //enum the child entries and everything underneath
-      List<FileStatus> childStats = listDirectory(srcObject, true, true, true);
       logDirectory("Directory to copy ", srcObject, childStats);
 
       // iterative copy of everything under the directory.
@@ -855,20 +780,11 @@ public class SwiftNativeFileSystemStore {
                   + "; copyDestSubPath=" + copyDestSubPath
                   + "; copyDestPath=" + copyDestPath);
         }
-        SwiftObjectPath copySource;
-        SwiftObjectPath copyDestination;
-
-        if (fileStatus.isDir()) {
-          copySource = toDirPath(copySourcePath);
-          copyDestination = toDirPath(copyDestPath);
-        } else {
-          copySource = toObjectPath(copySourcePath);
-          copyDestination = toObjectPath(copyDestPath);
-        }
+        SwiftObjectPath copyDestination = toObjectPath(copyDestPath);
 
         try {
-          copyThenDeleteObject(copySource, (SwiftFileStatus)fileStatus,
-                               copyDestination);
+          copyThenDeleteObject(toObjectPath(copySourcePath),
+                  copyDestination);
         } catch (FileNotFoundException e) {
           LOG.info("Skipping rename of " + copySourcePath);
         }
@@ -878,9 +794,8 @@ public class SwiftNativeFileSystemStore {
       //now rename self. If missing, create the dest directory and warn
       if (!SwiftUtils.isRootDir(srcObject)) {
         try {
-          if (!srcMetadata.isPseudoDir()) {
-            copyThenDeleteObject(srcObject, srcMetadata, targetObjectPath);
-          }
+          copyThenDeleteObject(srcObject,
+                  targetObjectPath);
         } catch (FileNotFoundException e) {
           //create the destination directory
           LOG.warn("Source directory deleted during rename", e);
@@ -908,6 +823,13 @@ public class SwiftNativeFileSystemStore {
     }
   }
 
+  public void copy(Path srcKey, Path dstKey) throws IOException {
+    SwiftObjectPath srcObject = toObjectPath(srcKey);
+    SwiftObjectPath destObject = toObjectPath(dstKey);
+    swiftRestClient.copyObject(srcObject, destObject);
+  }
+
+
   /**
    * Copy an object then, if the copy worked, delete it.
    * If the copy failed, the source object is not deleted.
@@ -918,74 +840,33 @@ public class SwiftNativeFileSystemStore {
 
    */
   private void copyThenDeleteObject(SwiftObjectPath srcObject,
-                                    SwiftFileStatus srcMeta,
                                     SwiftObjectPath destObject) throws
           IOException {
 
+
     //do the copy
-    copyObject(srcObject, srcMeta, destObject, true);
+    copyObject(srcObject, destObject);
+    //getting here means the copy worked
+    swiftRestClient.delete(srcObject);
   }
   /**
    * Copy an object
    * @param srcObject  source object path
-   * @param srcMeta    source object status
    * @param destObject destination object path
-   * @param deleteSrc  if true and the copy worked, delete source object
    * @throws IOException IO problems
    */
   private void copyObject(SwiftObjectPath srcObject,
-                          SwiftFileStatus srcMeta,
-                          SwiftObjectPath destObject,
-                          boolean deleteSrc) throws
+                                    SwiftObjectPath destObject) throws
           IOException {
     if (srcObject.isEqualToOrParentOf(destObject)) {
       throw new SwiftException(
         "Can't copy " + srcObject + " onto " + destObject);
     }
-    if (srcMeta.isDLO()) {
-      Path newPrefixPath = getCorrectSwiftPath(destObject);
-      String newPrefixName = newPrefixPath.toUri().getPath();
-      if (!newPrefixName.endsWith("/")) {
-        newPrefixName = newPrefixName.concat("/");
-      }
-      String oldPrefixName = srcMeta.getDLOPrefix().getObject();
-      List<FileStatus> segments = listDirectory(srcMeta.getDLOPrefix(),
-                                                true, true, true);
-      createManifestForPartUpload(newPrefixPath);
-      for (FileStatus s : segments) {
-        if (s.getLen() == 0) {
-          continue;
-        }
-        String oldName = s.getPath().toUri().getPath();
-        String newName = newPrefixName
-          + oldName.substring(oldPrefixName.length());
-        SwiftObjectPath srcSeg = new SwiftObjectPath(srcObject.getContainer(),
-                                                     oldName);
-        SwiftObjectPath destSeg = new SwiftObjectPath(destObject.getContainer(),
-                                                      newName);
-        if (!swiftRestClient.copyObject(srcSeg, destSeg)) {
-          throw new SwiftException("Copy of " + srcSeg + " to "
-                                 + destSeg + "failed");
-        }
-      }
-      if (deleteSrc) {
-        for (FileStatus s : segments) {
-          SwiftObjectPath srcSeg =
-            new SwiftObjectPath(srcObject.getContainer(),
-                                s.getPath().toUri().getPath());
-          swiftRestClient.delete(srcSeg);
-        }
-      }
-    } else {
-      //do the copy
-      boolean copySucceeded = swiftRestClient.copyObject(srcObject, destObject);
-      if (!copySucceeded) {
-        throw new SwiftException("Copy of " + srcObject + " to "
-                                 + destObject + "failed");
-      }
-    }
-    if (deleteSrc) {
-      swiftRestClient.delete(srcObject);
+    //do the copy
+    boolean copySucceeded = swiftRestClient.copyObject(srcObject, destObject);
+    if (!copySucceeded) {
+      throw new SwiftException("Copy of " + srcObject + " to "
+              + destObject + "failed");
     }
   }
 
@@ -1127,12 +1008,14 @@ public class SwiftNativeFileSystemStore {
     //don't mind if the directory has changed
     //list all entries under this directory.
     //this will throw FileNotFoundException if the file isn't there
-    FileStatus[] statuses;
-    try {
-      statuses = listSubPaths(absolutePath, true, askForNewest);
-    } catch (IOException e) {
-      // absolutePath is nonexistent
-      statuses = new FileStatus[0];
+    FileStatus[] statuses = listSubPaths(absolutePath, true, askForNewest);
+    if (statuses == null) {
+      //the directory went away during the non-atomic stages of the operation.
+      // Return false as it was not this thread doing the deletion.
+      SwiftUtils.debug(LOG, "Path '%s' has no status -it has 'gone away'",
+                       absolutePath,
+                       recursive);
+      return false;
     }
     int filecount = statuses.length;
     SwiftUtils.debug(LOG, "Path '%s' %d status entries'",
@@ -1141,7 +1024,7 @@ public class SwiftNativeFileSystemStore {
 
     if (filecount == 0) {
       //it's an empty directory or a path
-      deleteObject(fileStatus);
+      rmdir(absolutePath);
       return true;
     }
 
@@ -1153,14 +1036,14 @@ public class SwiftNativeFileSystemStore {
       // 1 entry => simple file and it is the target
       //simple file: delete it
       SwiftUtils.debug(LOG, "Deleting simple file %s", absolutePath);
-      deleteObject(fileStatus);
+      deleteObject(absolutePath);
       return true;
     }
 
     //>1 entry implies directory with children. Run through them,
     // but first check for the recursive flag and reject it *unless it looks
     // like a partitioned file (len > 0 && has children)
-    if (!fileStatus.isDirectory()) {
+    if (!fileStatus.isDir()) {
       LOG.debug("Multiple child entries but entry has data: assume partitioned");
     } else if (!recursive) {
       //if there are children, unless this is a recursive operation, fail immediately
@@ -1174,7 +1057,7 @@ public class SwiftNativeFileSystemStore {
     for (FileStatus entryStatus : statuses) {
       Path entryPath = entryStatus.getPath();
       try {
-        boolean deleted = deleteObject(entryStatus);
+        boolean deleted = deleteObject(entryPath);
         if (!deleted) {
           SwiftUtils.debug(LOG, "Failed to delete entry '%s'; continuing",
                            entryPath);
@@ -1189,38 +1072,8 @@ public class SwiftNativeFileSystemStore {
     }
     //now delete self
     SwiftUtils.debug(LOG, "Deleting base entry %s", absolutePath);
-    deleteObject(fileStatus);
+    deleteObject(absolutePath);
 
     return true;
   }
-
-  /**
-   * List all segments in dynamic large object.
-   *
-   * @param file   SwiftFileStatus of large object
-   * @param newest ask for the newest, or can some out of date data work?
-   * @return the file statuses, or an empty array if there are no segments
-   * @throws IOException           on IO problems
-   */
-  public FileStatus[] listSegments(FileStatus file, boolean newest)
-      throws IOException {
-    SwiftObjectPath prefix = ((SwiftFileStatus)file).getDLOPrefix();
-    if (prefix == null) {
-      return new FileStatus[0];
-    }
-
-    final List<FileStatus> objects;
-    objects = listDirectory(prefix, true, newest, false);
-
-    final ArrayList<FileStatus> segments;
-    segments = new ArrayList<FileStatus>(objects.size());
-
-    for (FileStatus status : objects) {
-      if (!status.isDir()) {
-        segments.add(status);
-      }
-    }
-
-    return segments.toArray(new FileStatus[segments.size()]);
-  }
 }
